using System;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.IO;
using System.Collections;
using System.Text;
// 
//  This code implements the Chair class and shows how to override
//  System.Object methods.
//
class Serial
{
	public static void TestSerialize()
	{
		// Test serialization
		Hashtable ht = new Hashtable();
		Chair ch = new Chair(100.00D, "Broyhill", "10-009");
		ch.myUpholstery = new Upholstery("Cotton");
		ht.Add("cotton", ch);
		// Add second item to table
		ch = new Chair(200.00D, "Lane", "10-99");
		ch.myUpholstery = new Upholstery("Silk");
		ht.Add("silk", ch);
		ch = new Chair(350.00D, "Henkel Harris", "09-08");
		ch.myUpholstery = new Upholstery("silk");
		ht.Add("silk2",ch);
		// Serialize
		// Create a new file; if file exits it is overwritten
		FileStream fs= new  FileStream("\\chairs.dat",FileMode.Create); 
		BinaryFormatter bf= new BinaryFormatter();
		bf.Serialize(fs,ht);
		fs.Close();
		// Deserialize binary file into a Hashtable of objects
		ht.Clear();
		fs = new FileStream("\\chairs.dat", FileMode.Open);
		ht =  (Hashtable) bf.Deserialize(fs);
		ch = (Chair)ht["silk"];
		Console.WriteLine("Deserialized Values:");
		Console.WriteLine(ch.myUpholstery.Fabric);
		Console.WriteLine(ch.MyVen);
		fs.Close();
	}
 

	public static void TestSorting()
	{
		Chair[]chairsOrdered = new Chair[4];
		chairsOrdered[0] = new Chair(150.0, "Lane","99-88");
		chairsOrdered[1] = new Chair(250.0, "Lane","99-00");
		chairsOrdered[2] = new Chair(100.0, "Lane","98-88");
		chairsOrdered[3] = new Chair(120.0, "Harris","93-9");
		Array.Sort(chairsOrdered);
		// Lists in ascending order of price
					foreach(Chair c in chairsOrdered) 
						Console.WriteLine(c.ToString());
	}
	
    public static void TestHashTable()
    {
       // Create HashTable
	   Hashtable chairHash = new Hashtable();
	   // Add key - value pair to Hashtable
	   chairHash.Add ("88-00", new Chair(350.0, "Adams", "88-00"));
	   chairHash.Add ("99-03", new Chair(380.0, "Lane", "99-03"));
	   // or this syntax
	   chairHash["89-01"] = new Chair(250.0, "Broyhill", "89-01");

       // (1) Check for existence of a Key
	   bool keyFound;
	   if (chairHash.ContainsKey("88-00"))
		{ keyFound = true;} 
	   else 
       {keyFound = false;}
       Console.WriteLine("{0} 88-00 in table",keyFound);
       
       // (2) List Keys
       Console.WriteLine("Hash Keys");
 		foreach (string invenKey in chairHash.Keys)
 			{ Console.WriteLine(invenKey); }
	   // (3) List keys and values
	   foreach ( DictionaryEntry deChair in chairHash)  
	   {
			Chair obj = (Chair) deChair.Value;
			Console.WriteLine(deChair.Key+" "+ obj.MyVen);
	   }

    
    }
}

// Entry point to application
class App
{     
    static void Main(string[] args)
    {  
		Serial.TestSerialize();
		Console.WriteLine("Sort by Price");
		Serial.TestSorting();	
		Serial.TestHashTable();
    }
}


[Serializable]
public class Chair: IComparable , ICloneable
{
	private double myPrice;
	private string myVendor, myID;
	private string finish;
	public Upholstery myUpholstery;
	// Constructor
	public Chair(double price, string vendor, string sku)
	{
		myPrice = price;
		myVendor = vendor;
		myID = sku;
	}

    // Override Equals() method
  	public override bool Equals(Object obj)
	{
	    // Include the following two statements if this class
	    // derives from a class that overrides Equals()
	    //if (!base.Equals(obj))
	    //	return false;
	    // (1) Null objects cannot be compared
	    if (obj == null) return false;
	    // (2) Check object types
	    if (this.GetType() != obj.GetType()) return false;
	    // (3) Cast object so we can access its fields.
	    Chair otherObj = (Chair) obj;
          // (4) Compare reference fields
    	if (!Object.Equals(myUpholstery, otherObj.myUpholstery)) return false;
	    // (5) Compare Value Type members
	    if (!myVendor.Equals(otherObj.myVendor)) return false;
	    if (!myPrice.Equals(otherObj.myPrice))   return false;
	    if (!myID.Equals(otherObj.myID))         return false;			    return true;
	}
	// Override GetHashCode - Required if Equals overridden
	public override int GetHashCode()
	{
	     return myID.GetHashCode();	 
 	}
    
    // Override ToString()
	public override string ToString()
	{
		StringBuilder chairSB = new StringBuilder();
		chairSB.AppendFormat("ITEM = Chair");
		chairSB.AppendFormat(" VENDOR = {0}", this.myVendor);
		chairSB.AppendFormat(" PRICE = {0}", 
 				this.myPrice.ToString());
		return chairSB.ToString();
	}
	
	// Override Clone
	public Object Clone()
	{
			return MemberwiseClone(); // from System.Object
	}	

 	// Override CompareTo()
	int IComparable.CompareTo(Object obj)
	{
		if (obj is Chair) 
		{
			Chair castObj = (Chair)obj;
			if (this.myPrice > castObj.myPrice)
					return 1;
			if (this.myPrice < castObj.myPrice)
					return -1;
			else return 0;
			// Reverse 1 and -1 to sort in descending order.
		}
		throw new ArgumentException("object in not a Chair");
	}

	public string MyVen
	{
		set { myVendor = value; }
		get { return myVendor; }

	}
	public double MyPr
	{
		get { return myPrice; }
	}
}
[Serializable]
public class Upholstery
{
	public string Fabric ;
	public Upholstery( string fab)
	{ Fabric = fab; }
}