using System;
// Custom Exception Class
[Serializable]
public class NoDescException : ApplicationException
{  // Three constructors should be implemented
	public NoDescException(){}
	public NoDescException(string message):base(message){}
	public NoDescException(string message, Exception innerEx)
	:base(message, innerEx){ }
}
	
// Interfaces that shape objects are to implement
public interface IShapeFunction
	{ double GetArea(); }
public interface IShapeDescription
	{ string ShowMe();}

// Circle and Rectangle classes are defined
class Circle : IShapeFunction
{
	private double radius;
	public Circle (double rad)
	{
    	radius= rad;
	}
	// Methods to implement both interfaces
	public double GetArea()
	{  return (3.14*radius*radius);  }
}
class Rectangle : IShapeFunction , IShapeDescription 
{
	private int width, height;
	public Rectangle(int w, int h)
	{
		width= w;
		height=h;
	}
	// Methods to implement both interfaces
	public double GetArea()
	{  return (height*width);  }
	public string ShowMe()
	{  return("rectangle");    }
}
	
public class ObjAreas
{
	public static void ShowAreas(object ObjShape)
	{	
		// Check to see if interfaces are implemented
		if (!(ObjShape is IShapeDescription && 
		ObjShape is IShapeFunction) )
		{  
			// throw custom exception. 
			string objName = ObjShape.ToString();
			throw new NoDescException  
				("Interface not implemented for "+objName);
		}  
		// Continue Processing since interfaces exist
		IShapeFunction myShape   = (IShapeFunction)ObjShape;
		IShapeDescription myDesc = (IShapeDescription) ObjShape;
		string desc = myDesc.ShowMe();
		Console.WriteLine(desc+" Area= "+
		myShape.GetArea().ToString());
	}
} 

class MyApp 
{
	public static void Main() 
	{ 
		 Circle myCircle = new Circle(4.0);
		 Rectangle myRect = new Rectangle(5,2);
		 try 
		 {
			 ObjAreas.ShowAreas(myRect);
			 ObjAreas.ShowAreas(myCircle);
		 }
		 catch (NoDescException ex) 
		 {
			 Console.WriteLine(ex.Message);
		 }	
	}
}	